'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.mapToRelative = mapToRelative;
exports.mapModule = mapModule;
var path = require('path');

var fs = require('fs');

var dirContentsMap = {};

function createFilesMap(state) {
  var result = {};
  var opts = Array.isArray(state.opts) ? state.opts : [state.opts];

  opts.forEach(function (moduleMapData) {
    result[moduleMapData.expose] = moduleMapData.src;
  });

  return result;
}

function resolve(filename) {
  if (path.isAbsolute(filename)) return filename;
  return path.resolve(process.cwd(), filename);
}

function toPosixPath(modulePath) {
  return modulePath.replace(/\\/g, '/');
}

function pathFix(os, base, fileList) {
  // throw new Error(`${os} | ${base} | ${fileList}`);
  // Check for mobile substitutions first
  if (['mobile', 'ios', 'android', 'windows'].indexOf(os) > -1) {
    // Check for a mobile file first
    if (fileList.indexOf(base + '.mobile.js') > -1) {
      return base + '.mobile';
    }

    // Check for OS specific file
    if (fileList.indexOf(base + '.' + os + '.js') > -1) {
      return base + '.' + os;
    }

    // Fallback on normal file (web/desktop) for passthrow files
    return pathFix('desktop', base, fileList);
  } else if (os === 'desktop') {
    // Check for desktop only files
    if (fileList.indexOf(base + '.desktop.js') > -1) {
      return base + '.desktop';
    }

    // Fallback on web
    return pathFix('web', base, fileList);
  } else if (os === 'web') {
    if (fileList.indexOf(base + '.web.js') > -1) {
      return base + '.web';
    }
  }

  // Suffixless files
  if (fileList.indexOf(base + '.js') > -1) {
    return '' + base;
  }

  return base;
}

function mapToRelative(currentFile, module, supportReactNative) {
  var from = path.dirname(currentFile);
  var to = path.normalize(module);

  var osENV = (process.env.REACT_NATIVE_ENV || '').toLowerCase();

  from = resolve(from);
  to = resolve(to);

  var moduleMapped = path.relative(from, to);

  moduleMapped = toPosixPath(moduleMapped);

  // Support npm modules instead of directories
  if (moduleMapped.indexOf('npm:') !== -1) {
    var _moduleMapped$split = moduleMapped.split('npm:');

    var _moduleMapped$split2 = _slicedToArray(_moduleMapped$split, 2);

    var npmModuleName = _moduleMapped$split2[1];

    return npmModuleName;
  }

  if (moduleMapped[0] !== '.') moduleMapped = './' + moduleMapped;

  // Support React-Native specific require rewrites
  if (process.env.REACT_NATIVE && supportReactNative) {
    var base = path.dirname(moduleMapped);

    // Index files in destination directory once
    if (!dirContentsMap[base]) {
      dirContentsMap[base] = fs.readdirSync(base).map(function (v) {
        return v.toLowerCase();
      });
    }

    // Fix mapped module path
    var newFile = pathFix(osENV, path.basename(moduleMapped, 'js'), dirContentsMap[base]);

    return base + '/' + newFile;
  }
  // }

  return moduleMapped;
}

function mapModule(source, file, filesMap) {
  var supportReactNative = false;
  var s = source;

  // Allow React-Native automatic file detection when applies
  if (process.env.REACT_NATIVE) {
    var aux = source.toLowerCase().indexOf('autoimport:');

    if (aux > -1) {
      s = source.slice(supportReactNative + 11);
      supportReactNative = true;
    }
  }

  var moduleSplit = s.split('/');

  var src = void 0;
  while (moduleSplit.length) {
    var m = moduleSplit.join('/');
    if ({}.hasOwnProperty.call(filesMap, m)) {
      src = filesMap[m];
      break;
    }
    moduleSplit.pop();
  }

  if (!moduleSplit.length) {
    // no mapping available
    return null;
  }

  var newPath = s.replace(moduleSplit.join('/'), src);
  return mapToRelative(file, newPath, supportReactNative);
}

exports.default = function (_ref) {
  var t = _ref.types;

  function transformRequireCall(nodePath, state) {
    if (!t.isIdentifier(nodePath.node.callee, { name: 'require' }) && !(t.isMemberExpression(nodePath.node.callee) && t.isIdentifier(nodePath.node.callee.object, { name: 'require' }))) {
      return;
    }

    var moduleArg = nodePath.node.arguments[0];
    if (moduleArg && moduleArg.type === 'StringLiteral') {
      var filesMap = createFilesMap(state);
      var modulePath = mapModule(moduleArg.value, state.file.opts.filename, filesMap);
      if (modulePath) {
        nodePath.replaceWith(t.callExpression(nodePath.node.callee, [t.stringLiteral(modulePath)]));
      }
    }
  }

  function transformImportCall(nodePath, state) {
    var moduleArg = nodePath.node.source;
    if (moduleArg && moduleArg.type === 'StringLiteral') {
      var filesMap = createFilesMap(state);
      var modulePath = mapModule(moduleArg.value, state.file.opts.filename, filesMap);
      if (modulePath) {
        nodePath.replaceWith(t.importDeclaration(nodePath.node.specifiers, t.stringLiteral(modulePath)));
      }
    }
  }

  return {
    visitor: {
      CallExpression: {
        exit: function exit(nodePath, state) {
          return transformRequireCall(nodePath, state);
        }
      },
      ImportDeclaration: {
        exit: function exit(nodePath, state) {
          return transformImportCall(nodePath, state);
        }
      }
    }
  };
};